#! /usr/bin/perl
#---------------------------------------------------------------------

use Test::More tests => 1;

BEGIN {
    use_ok('Dist::Zilla::Plugin::TemplateCJM');
}

diag("Testing Dist::Zilla::Plugin::TemplateCJM $Dist::Zilla::Plugin::TemplateCJM::VERSION");
